/**
 * 	-------------------------------------------------------------------------------------
 * 	Requirements
 * 	-------------------------------------------------------------------------------------
 */
var util    			= require('util');
var EventEmitter 		= require("events").EventEmitter;

var mysql 				= require('mysql');
var async   			= require('async');

/**	-------------------------------------------------------------------------------------
 * 	Class CDBS
 *	
 * 	version: 		1.0
 * 	description: 	- Database Managemant 
 * 					- multiple MysQL database pools 
 * 	-------------------------------------------------------------------------------------
 */ 
function CDBS(databases){
	var self = this;

	self.init = function(version){
        //  Verbindungen zu hinterlegten Datenbanken aufbauen:
        async.each(databases,	
			function _connectDB(options, next) {  
				var dbPool = mysql.createPool(options);
               	dbPool.insert = _insert;
               	dbPool.delete = _delete;
               	dbPool.update = _update;
               	dbPool.select = _select;
               	dbPool.on('connection', 	_connectionEventHandler);
               	dbPool.on('enqueue', 		_enqueueEventHandler);
               	
               	if(!version){
                    dbPool.query('SHOW TABLES', function(err, results, fields) {
                        if (err) { if (err) { return next(err); }; }
                        //  Zu jedem Table die Referenz speichern:
                        results.forEach(function(subset) {
                            self[subset[fields[0].name]] = dbPool;   //  e.g. dbs["TABLE_NAME"].query(...)
                        });
                        next();                 
                    });                 	    
               	}else{
               	    self[options.dbsName] = dbPool;
               	    next();
               	}      
            },
            function _connectDBFinish(err) {
            	if(err) { return self.emit("error", err); }
				self.emit("ready");
            }
        );
	};
	
	var _connectionEventHandler = function(event){
		self.emit("connection", event);
	};
		
	var _enqueueEventHandler = function(event){
		self.emit("enqueue", event);
	};
	
	var _insert = function(sql, params, callback){
		this.query(sql, params, function(err, result){
			if (err) { return callback(err, null); }
			if(callback) callback(null, result);
		});
	};
	
	var _delete = function(sql, params, callback){
		this.query(sql, params, function(err, result){
			if (err) { return callback(err, null); }
			if(callback) callback(null, result);
		});
	};
	
	var _update = function(sql, params, callback){
		this.query(sql, params, function(err, result){
			if (err) { return callback(err, null); }
			if(callback) callback(null, result);
		});
	};
	
	var _select = function(sql, params, callback){
		this.query(sql, params, function(err, result){
			if (err) { return callback(err, null); }
			if(callback) callback(null, result);
		});
	};
};

util.inherits(CDBS, EventEmitter);
module.exports = CDBS;